unit MainUnit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VirtualTrees, VirtualTreesHelper;

type
  TForm2 = class(TForm)
    treeMain: TVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure treeMainGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: string);
    procedure FormDestroy(Sender: TObject);
    procedure treeMainKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure treeMainMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure treeMainMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure treeMainMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure treeMainBeforeCellPaint(Sender: TBaseVirtualTree; TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure treeMainPaintText(Sender: TBaseVirtualTree; const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
  private
    { Private declarations }
    FVTSelection: TTreeSelection;
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

uses Winapi.UxTheme;

{$R *.dfm}

procedure TForm2.FormCreate(Sender: TObject);
begin
  treeMain.RootNodeCount := 20;
  FVTSelection := TTreeSelection.Create(treeMain);
end;

procedure TForm2.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FVTSelection);
end;

procedure TForm2.treeMainBeforeCellPaint(Sender: TBaseVirtualTree; TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);

var
  Theme: HTHEME;

begin
  if Sender is TVirtualStringTree  then
  begin
    if FVTSelection.IsSelectionActive then
    begin
      // draw selected cells
      if FVTSelection.IsInSelection(Node.Index, Column) then
      begin
        if IsWinVistaOrAbove then
        begin
          Theme := OpenThemeData(Application.ActiveFormHandle, 'Explorer::TreeView');
          DrawThemeBackground(Theme, TargetCanvas.Handle, TVP_TREEITEM, TREIS_HOTSELECTED, CellRect, nil);
        end
        else
        begin
          Brush.Color := TVirtualStringTree(Sender).Colors.FocusedSelectionColor;;
          TargetCanvas.FillRect(CellRect);
        end;
      end;
    end;
  end;
end;

procedure TForm2.treeMainGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: string);
begin
  CellText := IntToStr(Node.Index) +  Chr(Ord('A') + Column);
end;

procedure TForm2.treeMainKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  FVTSelection.KeyPress(Key, Shift);
end;

procedure TForm2.treeMainMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FVTSelection.MouseDown(X, Y, Shift);
end;

procedure TForm2.treeMainMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  FVTSelection.MouseMove(X, Y, Shift);
end;

procedure TForm2.treeMainMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FVTSelection.MouseUp(X, Y, Shift);
end;

procedure TForm2.treeMainPaintText(Sender: TBaseVirtualTree; const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if FVTSelection.IsSelectionActive and (not IsWinVistaOrAbove) then
  begin
    if FVTSelection.IsInSelection(Node.Index, Column) then
      TargetCanvas.Font.Color := clWhite;
  end;
end;

end.
